@@ -1,7 +1,7 @@
 """pavlovadm packaging information"""
 name = 'pavlovadm'
 provides = ['pavloavadm']
-version = '1.10'
+version = '1.15'
 install_requires = [
     'requests', 'inquirer', 'PyYAML']
 url = 'https://pypi.org/project/pavlovadm/'
@@ -38,4 +38,5 @@ package_data = {
 
 data_files=[
     ('share/pavlovadm', [
-        'pavlovadm/example/BalancingTable.csv', 'pavlovadm/example/Game.ini', 'pavlovadm/example/public.ini'])]
+        'pavlovadm/example/BalancingTable.csv', 'pavlovadm/example/Game.ini',
+        'pavlovadm/example/public.ini', 'pavlovadm/example/pavlovadm.conf'])]
