@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 from os import \
     makedirs, environ
@@ -40,12 +40,10 @@ from inquirer import prompt, List as iList
 
 from cmd import Cmd
 
-from pavlovadm.colortext import tabd
+from pavlovadm.colortext import tabd, error, bgre
 
 class PavlovADM(Cmd):
 	servers = {}
-	socket = sock(AF_INET, SOCK_STREAM)
-	socket.settimeout(3)
 	gameini = ''
 	itemtbl = ''
 	maptbl = '~/.cache/pavlovadm/maps.tbl'
@@ -53,45 +51,17 @@ class PavlovADM(Cmd):
 	maps = {}
 	hlp = None
 	cnt = 0
-	srv = None
+	dbg = None
+	pwd = None
 	def __init__(self, *args, **kwargs):
 		self.use_rawinput = False
 		for (k, v) in kwargs.items():
 			if hasattr(self, k):
 				setattr(self, k, v)
 		super().__init__()
-		try:
-			self.serverselect()
-		except IndexError as err:
-			print('\033c')
-			print('known index error occured, restarting')
-			self.serverselect()
-
-	def serverselect(self):
-		if not self.srv:
-			self.srv = self._login()
-		if not self.hlp:
-			while True:
-				hlp = self._send('Help')
-				if hlp:
-					break
-			self.hlp = hlp
-		cmd = self._cmdselects()
-		if cmd is None:
-			return self._cmdselects()
-		#print(cmd)
-		res = self.fire(cmd.strip('<>'))
-		if res:
-			print(res)
-		if cmd == '<Disconnect>':
-			self.socket.close()
-			self.srv = None
-			self.maps = {}
-			self.socket = sock(AF_INET, SOCK_STREAM)
 		self.serverselect()
 
-	def _login(self):
-		"""server login method"""
+	def serverselect(self):
 		ask = [
             iList(
                 'srv',
@@ -103,15 +73,17 @@ class PavlovADM(Cmd):
 		srv = list(prompt(ask).values())[0]
 		if srv == '<Exit>':
 			exit()
-		passwd = self.servers[srv][0]
-		if len(self.servers[srv]) > 1:
-			maplist = self.servers[srv][1]
-		server, port = srv.split(':')
-		self.socket.connect((server, int(port)))
-		self._send('')
-		print(self._send(passwd))
-		return srv
-		#self.prompt = '%s:%s >'%(server, port)
+		while True:
+			cmd = self._cmdselects(srv)
+			if not cmd:
+				continue
+			if cmd == '<Disconnect>':
+				self.maps = {}
+				break
+			res = self.rconexec(srv, cmd.strip('<>'))
+			if res:
+				print(tabd(res))
+		self.serverselect()
 
 	def _getmapname(self, mapid):
 		if mapid in self.mapnames:
@@ -122,11 +94,11 @@ class PavlovADM(Cmd):
 			if 'workshopItemTitle' in l:
 				return l.split('>')[1].split('<')[0]
 
-	def _getmaps(self, noask=None):
-		if not self.maps:
+	def _getmaps(self, srv, noask=None):
+		if not self.maps or not self.mapnames:
 			maplst = self.gameini
-			if len(self.servers[self.srv]) > 1:
-				maplst = self.servers[self.srv][1]
+			if len(self.servers[srv]) > 1:
+				maplst = self.servers[srv][1]
 			if maplst.startswith('~'):
 				maplst = expanduser(maplst)
 			if not maplst.startswith('/'):
@@ -150,7 +122,7 @@ class PavlovADM(Cmd):
 				self.mapnames[ugcid] = name
 			with open(expanduser(self.maptbl), 'w+') as mfh:
 				mfh.write(ydump(self.mapnames, Dumper=Dumper))
-			if noask: return
+		if noask: return self.mapnames
 		ask = [
           iList(
             'map',
@@ -243,12 +215,15 @@ class PavlovADM(Cmd):
 			else:
 				print('too dumb for numbers? o.0 aborting...')
 
-	def _cmdselects(self):
+	def _cmdselects(self, srv):
 		noargs = ['Info', 'ResetSND', 'RefreshList', 'RotateMap', 'ServerInfo', 'Help', '<Disconnect>']
 		steams = ['Kick', 'Ban', 'Unban', 'InspectPlayer']
 		others = ['SwitchMap', 'SwitchTeam', 'GiveItem', 'GiveCash', 'GiveTeamCash', 'SetPlayerSkin', 'SetLimitedAmmoType']
 		order = ['Info', 'SwitchMap', 'RotateMap', 'ResetSND', 'Kick', 'Ban', 'Unban', 'InspectPlayer', 'GiveItem', 'GiveCash', 'GiveTeamCash', 'SetPlayerSkin', 'SetLimitedAmmoType', 'RefreshList', 'ServerInfo', '<Disconnect>']
-		hlp = self.hlp['Help']
+		hlp = self.hlp if self.hlp else self.rconexec(srv, 'Help')
+		if not hlp:
+			error('no help', hlp)
+		hlp = hlp['Help']
 		hlp = [h.split(' ')[0] for h in hlp.split(', ') if h.split(' ')[0]] + ['Info']
 		ask = [
             iList(
@@ -262,28 +237,28 @@ class PavlovADM(Cmd):
 		if cmd in noargs:
 			return cmd
 		elif cmd in steams:
-			sid = self._getsteamid(cmd)
+			sid = self._getsteamid(srv, cmd)
 			if not sid:
 				return
 			return '%s %s'%(cmd, sid)
 		elif cmd in others:
 			if cmd == 'SwitchMap':
-				mapmod = self._getmaps()
+				mapmod = self._getmaps(srv)
 				if not mapmod:
 					return
 				return 'SwitchMap %s'%mapmod
 			elif cmd == 'SwitchTeam':
-				sid = self._getsteamid(cmd)
+				sid = self._getsteamid(srv, cmd)
 				if not sid:
 					return
 				return 'SwitchTeam %s %s'%(sid, self._getteam())
 			elif cmd == 'GiveItem':
-				sid = self._getsteamid(cmd)
+				sid = self._getsteamid(srv, cmd)
 				if not sid:
 					return
 				return 'GiveItem %s %s'%(sid, self._getitem())
 			elif cmd == 'GiveCash':
-				sid = self._getsteamid(cmd)
+				sid = self._getsteamid(srv, cmd)
 				if not sid:
 					return
 				return 'GiveCash %s %s'%(sid, self._getcash())
@@ -293,7 +268,7 @@ class PavlovADM(Cmd):
 					return
 				return 'GiveTeamCash %s %s'%(team, self._getcash())
 			elif cmd == 'SetPlayerSkin':
-				sid = self._getsteamid(cmd)
+				sid = self._getsteamid(srv, cmd)
 				if not sid:
 					return
 				return 'SetPlayerSkin %s %s'%(sid, self._getskin())
@@ -303,8 +278,8 @@ class PavlovADM(Cmd):
 					return
 				return 'SetLimitedAmmoType %s'%ammo
 
-	def _getsteamid(self, cmd):
-		userids = self._players()
+	def _getsteamid(self, srv, cmd):
+		userids = self._players(srv)
 		if not userids:
 			print('\nerror: executing "%s" is impossible - no players\n'%cmd)
 			return
@@ -321,21 +296,10 @@ class PavlovADM(Cmd):
 			return
 		return userids[usr]
 
-	def fire(self, cmd):
-		#print(cmd)
-		if cmd == 'Info':
-			if not self.maps:
-				self._getmaps(True)
-			l = max(len(k) for k in self.maps.keys())+1
-			msg = 'MapList:\n%s\n\n%s\n\n%s\n'%(
-			      '\n'.join('  %s%s= %s'%(k, ' '*int(l-len(k)), v) for (k, v) in self.maps.items()), \
-			      tabd(self._send('ServerInfo')), tabd(self._send('RefreshList')))
-			return msg
-		return self._send(cmd)
 
-	def _players(self):
-		pout = self._send('RefreshList')
-		return jload(pout)['PlayerList']
+	def _players(self, srv):
+		pout = dict(self.rconexec(srv, 'RefreshList'))['PlayerList']
+		return pout
 		#pout = pout.split('{\n\t"PlayerList": [')[1]
 		#pout = pout.split('\t]\n}')[0]
 		##print(pout)
@@ -347,36 +311,63 @@ class PavlovADM(Cmd):
 		#	_players[useruid[0].split('": "')[1].rstrip('",')] = useruid[1].split('": "')[1].rstrip('"')
 		#return _players
 
-	def _getresponse(self):
+	def _getresponse(self, socket):
 		res = []
 		while True:
-			ret = self.socket.recv(1024)
+			ret = socket.recv(1024)
 			res.append(ret.decode())
 			if len(ret) <= 1023:
 				break
 		return ''.join(res)
 
-	def _send(self, data, answer=True):
-		res = ''
+	def _send(self, srv, data):
+		passwd = self.servers[srv][0]
+		server, port = srv.split(':')
+		socket = sock(AF_INET, SOCK_STREAM)
+		socket.settimeout(3)
+		socket.connect((server, int(port)))
+		socket.sendall(''.encode())
+		auth = self._getresponse(socket)
+		socket.sendall(passwd.encode())
+		auth = self._getresponse(socket)
+		if str(auth).strip() != 'Authenticated=1':
+			error('authentication failure')
+		if self.dbg:
+			print(bgre(auth))
+		res = {}
 		try:
-			self.socket.sendall(data.encode())
-			if answer:
-				res = self._getresponse()
-				if not res:
-					self.socket.sendall(''.encode())
-					res = self._getresponse()
-			res = jload(res)
+			socket.sendall(data.encode())
+			res = self._getresponse(socket)
+			if res:
+				res = jload(res)
 		except JSONDecodeError as err:
-			print(err)
+			pass
 		except TimeOutError:
-			print('we ran into a timeout wile executing "%s"'%data, end='')
 			self.cnt += 1
-			if self.cnt <= 3:
-				print(' retrying...')
-				res = self._send(data, answer)
-			print(' aborting...')
+			if self.cnt <= 2:
+				print('we ran into a timeout wile executing "%s", retrying...'%data)
+				res = self._send(srv, data)
+			print('we ran into a timeout wile executing "%s", aborting...'%data)
 		finally:
-			return res
+			socket.close()
+		return res
+
+	def rconexec(self, srv, cmd):
+		if self.dbg:
+			print(bgre('%s %s'%(srv, cmd)))
+		if cmd == 'Info':
+			res = self._send(srv, 'ServerInfo')
+			res = res['ServerInfo']
+			res['MapName'] = self._getmapname(res['MapLabel'])
+			res['PlayerList'] = self._players(srv)
+			res['MapList'] = self._getmaps(srv, True)
+			res = {'Info': res}
+		else:
+			res = self._send(srv, cmd)
+		if isinstance(res, dict):
+			res = dict((k, v) for (k,v) in sorted(res.items()))
+		return res
+
 
 	#def default(self, line):
 	#       if line == 'Disconnect':
@@ -407,21 +398,31 @@ def main():
 		makedirs(cacdir)
 	except FileExistsError:
 		pass
-	src = '~/.local/share/pavlovadm'
+	src = expanduser('~/.local/share/pavlovadm')
+	lsrc = '/usr/local/share/pavlovadm'
 	if not isdir(src):
-		src = '/usr/local/share/pavlovadm'
+		src = lsrc
 	if isdir(src):
-		for f in ('~/.local/share/pavlovadm/pavlovadm.conf', '~/.local/share/pavlovadm/Game.ini', '~/.local/share/pavlovadm/public.ini', '~/.local/share/pavlovadm/BalancingTable.csv'):
+		for f in ('%s/pavlovadm.conf'%src, '%s/Game.ini'%src, '%s/public.ini'%src, '%s/BalancingTable.csv'%src):
+			if not isfile(f):
+				f = '%s/%s'%(lsrc, basename(f))
+				if not isfile(f):
+					continue
 			if not isfile('%s/%s'%(cfgdir, basename(f))):
-				with open(expanduser(f), 'r') as lfh, open(expanduser('~/%s/%s'%(cfgdir, basename(f))), 'w+') as gfh:
+				with open(expanduser(f), 'r') as lfh, open('%s/%s'%(cfgdir, basename(f)), 'w+') as gfh:
 					gfh.write(lfh.read())
-	cfgs = config(expanduser('~/%s/%s.conf'%(__me, __me)))
+	cfgs = config('%s/%s.conf'%(cfgdir, __me))
 
 	pars = ArgumentParser(description=__dsc)
 	pars.add_argument(
         '--version',
         action='version', version='%s v0.13'%__me)
+	pars.add_argument(
+        '-D', '--debug',
+        dest='dbg', action='store_true', help='enable debugging')
 	args = pars.parse_args()
+	if args.dbg:
+		cfgs['dbg'] = True
 	cli(cfgs)
 
 
